/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.actions;

import java.io.File;

import com.eviware.soapui.impl.WorkspaceImpl;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.support.AbstractSoapUIAction;
import com.eviware.x.form.XFormDialog;
import com.eviware.x.form.XFormField;
import com.eviware.x.form.XFormFieldListener;
import com.eviware.x.form.support.ADialogBuilder;
import com.eviware.x.form.support.AField;
import com.eviware.x.form.support.AForm;
import com.eviware.x.form.support.AField.AFieldType;

/**
 * Action for creating a new WSDL project
 * 
 * @author Ole.Matzura
 */

public class NewWsdlProjectAction extends AbstractSoapUIAction<WorkspaceImpl>
{
	public static final String SOAPUI_ACTION_ID = "NewWsdlProjectAction";
	private XFormDialog dialog;

	public NewWsdlProjectAction()
   {
      super("New WSDL Project", "Creates a new WSDL Project in this workspace"); 
   }
   
	public void perform( WorkspaceImpl workspace, Object param )
	{
   	if( dialog == null )
   	{
			dialog = ADialogBuilder.buildDialog( Form.class );
			dialog.setValue( Form.CREATEREQUEST, Boolean.toString( true ) );
			dialog.getFormField( Form.INITIALWSDL ).addFormFieldListener( new XFormFieldListener(){

				public void valueChanged( XFormField sourceField, String newValue, String oldValue )
				{
					dialog.getFormField( Form.CREATEREQUEST ).setEnabled( newValue.trim().length() > 0 );
				}} );
   	}
   	else 
   	{
   		dialog.setValue( Form.PROJECTNAME, "" );
   		dialog.setValue( Form.INITIALWSDL, "" );
   	}
   	
   	if( dialog.show() )
   	{
   		try
   		{
   			WsdlProject project = workspace.createProject( dialog.getValue( Form.PROJECTNAME ));
   			if( project != null )
            {
   				String url = dialog.getValue( Form.INITIALWSDL ).trim();
   				if( url.length() > 0 )
   				{
   					if( new File( url ).exists() )
   						url = "file:" + url;
   					
      				project.importWsdl( url, dialog.getValue( Form.CREATEREQUEST ).equals( "true" ));
   				}
   				
   			   UISupport.select(project);
            }
   		}
   		catch (Exception ex)
   		{
   			UISupport.showErrorMessage( ex );
   		}
   	}
	}
   
   @AForm( name="New WSDL Project", description = "Creates a new WSDL Project in this workspace")
	private class Form 
	{
		@AField(name = "Project Name", description = "The name of the project to create", type = AFieldType.STRING )
		public final static String PROJECTNAME = "Project Name";
		
		@AField(name = "Initial WSDL", description = "URL or filename of initial WSDL", type = AFieldType.FILE )
		public final static String INITIALWSDL = "Initial WSDL";
		
		@AField(name = "Create Requests", description = "Create sample requests for all operations?", 
					type = AFieldType.BOOLEAN, enabled = false )
		public final static String CREATEREQUEST = "Create Requests";
	}

	
}