/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.actions;

import java.io.File;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.WorkspaceImpl;
import com.eviware.soapui.support.SoapUIException;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.support.AbstractSoapUIAction;

/**
 * Action to swtich the current workspace
 * 
 * @author ole.matzura
 */

public class SwitchWorkspaceAction extends AbstractSoapUIAction<WorkspaceImpl>
{
	public static final String SOAPUI_ACTION_ID = "SwitchWorkspaceAction";

	public SwitchWorkspaceAction()
	{
		super( "Switch Workspace", "Switch to another workspace file" );
	}
	
	public void perform( WorkspaceImpl workspace, Object param )
	{
		if( SoapUI.getTestMonitor().hasRunningTests() )
		{
			UISupport.showErrorMessage( "Cannot switch workspace white tests are running" );
			return;
		}
		
		File newPath = UISupport.getFileDialogs().open( this, "Switch Workspace", ".xml", "soapUI Workspace (*.xml)", 
					workspace.getPath() );
		
		if( newPath != null )
		{
			if( SoapUI.getDesktop().closeAll())
			{
				boolean save = true;
				
				if( !newPath.exists() )
				{
					if( !UISupport.confirm( "Create new Workspace in file [" + newPath.getName() + "]", 
								"Switch Workspace" ))
					{
						return;
					}			
					
					save = false;
				}
				else if( workspace.getProjectCount() > 0 )
				{
					Boolean val = UISupport.confirmOrCancel( "Save All Projects before Switching Workspace?", "Switch Workspace" );
					if( val == null )
						return;
					
					save = val.booleanValue();
				}
				
				workspace.save( !save );
				
				try
				{
					workspace.changeWorkspace( newPath );
					SoapUI.getSettings().setString( SoapUI.CURRENT_SOAPUI_WORKSPACE, newPath.getAbsolutePath() );
					UISupport.select( workspace );
				}
				catch( SoapUIException e )
				{
					UISupport.showErrorMessage( e );
				}
			}
		}
	}
}