/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.iface;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;

import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequest;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.model.iface.Request;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.testsuite.TestSuite;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.support.AbstractSoapUIAction;
import com.jgoodies.forms.builder.ButtonBarBuilder;

/**
 * Manages the service endpoints for a WsdlInterface
 * 
 * @author Ole.Matzura
 */

public class InterfaceEndpointsAction extends AbstractSoapUIAction<WsdlInterface>
{
   private JDialog dialog;
   private JList list;
   private DefaultListModel listModel;
	private WsdlInterface iface;
   private final static Logger log = Logger.getLogger( InterfaceEndpointsAction.class );
	private JButton editButton;
	private JButton deleteButton;
	private JButton assignButton;

   public InterfaceEndpointsAction()
   {
      super( "Service Endpoints", "Manage service endpoints available for this interface" );
   }

	private void buildDialog()
	{
		dialog = new JDialog( UISupport.getMainFrame() );
      dialog.setTitle("Interface Service Endpoints" );
      
      JPanel contentPanel = new JPanel( new BorderLayout() );
      listModel = new DefaultListModel();
      list = new JList( listModel );
      list.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
      list.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
      list.getSelectionModel().addListSelectionListener( new ListSelectionListener() {

			public void valueChanged( ListSelectionEvent e )
			{
				enableButtons();
			}} );
      
      JScrollPane scrollPane = new JScrollPane( list );
      
      scrollPane.setBorder( BorderFactory.createCompoundBorder(
      		BorderFactory.createEmptyBorder( 5, 5, 5, 5 ), BorderFactory.createLineBorder( Color.GRAY )));
      
      scrollPane.setVerticalScrollBarPolicy( ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS );
      
      contentPanel.add( scrollPane, BorderLayout.CENTER );
      contentPanel.add( createButtons(), BorderLayout.SOUTH );
      Component descriptionPanel = UISupport.buildDescription( "Service Endpoints", 
      			"Edit available service endpoints for this interface in list below", null );
      contentPanel.add( descriptionPanel, BorderLayout.NORTH );
      
      dialog.setContentPane( contentPanel );
      dialog.setSize(400, 300);
      
      dialog.setModal( true );
	}
	
   protected void enableButtons()
	{
		editButton.setEnabled( list.getSelectedIndex() != -1 );
		deleteButton.setEnabled( list.getSelectedIndex() != -1 );
		assignButton.setEnabled( list.getSelectedIndex() != -1 );
	}

	public void perform( WsdlInterface iface, Object param )
	{
   	if( dialog == null )
   		buildDialog();
   	
      listModel.clear();
      
      String[] endpoints = iface.getEndpoints();
      for( int c = 0; c < endpoints.length; c++ )
      {
         listModel.addElement( endpoints[c] );
      }
      
      list.setSelectedIndex( -1 );
      
      this.iface = iface;
      enableButtons();
      UISupport.showDialog( dialog );
   }

   private Component createButtons()
   {
      ButtonBarBuilder builder = ButtonBarBuilder.createLeftToRightBuilder();
		builder.addFixed( UISupport.createToolbarButton( new ShowOnlineHelpAction( HelpUrls.ENDPOINTSEDITOR_HELP_URL )));
		builder.addGlue();
		builder.addFixed( new JButton( new AddAction() ));
      builder.addRelatedGap();
      editButton = new JButton( new EditAction() );
		builder.addFixed( editButton);
      builder.addRelatedGap();
      deleteButton = new JButton( new DeleteAction() );
		builder.addFixed( deleteButton);
      builder.addRelatedGap();
      assignButton = new JButton( new AssignAction() );
		builder.addFixed( assignButton);
      builder.addRelatedGap();
      builder.addFixed( new JButton( new OkAction() ));
      
      builder.setBorder(
   				BorderFactory.createCompoundBorder( 
   						BorderFactory.createCompoundBorder( 
   								BorderFactory.createMatteBorder( 1, 0, 0, 0, Color.GRAY ),
   								BorderFactory.createMatteBorder( 1, 0, 0, 0, Color.WHITE )),
   						BorderFactory.createEmptyBorder( 3, 5, 3, 5 )));
      
      return builder.getPanel();
   }
   
   private class AddAction extends AbstractAction
   {
      public AddAction()
      {
         super( "Add" );
         putValue( Action.SHORT_DESCRIPTION, "Adds a new endpoint to the list" );
      }
      
      public void actionPerformed(ActionEvent e)
      {
         Object selectedValue = list.getSelectedValue();
			String endpoint = UISupport.prompt( "Enter new endpoint URL", "Add Endpoint", 
         		selectedValue == null ? "" : selectedValue.toString() );
			
         if( endpoint == null ) return;
         
         listModel.addElement( endpoint );
         iface.addEndpoint( endpoint );
      }
   }
   
   private class AssignAction extends AbstractAction
   {
      private static final String ALL_REQUESTS = "- All Requests -";
      private static final String ALL_TEST_REQUESTS = "- All Test Requests -";
      private static final String ALL_REQUESTS_AND_TEST_REQUESTS = "- All Requests and TestRequests -";
		private static final String ALL_REQUESTS_WITH_NO_ENDPOINT = "- All Requests with no endpoint -";

		public AssignAction()
      {
         super( "Assign" );
         putValue( Action.SHORT_DESCRIPTION, "Assigns the selected endpoint to Requests/TestRequests for this Interface" );
      }
      
      public void actionPerformed(ActionEvent e)
      {
      	int selectedIndex = list.getSelectedIndex();
         if( selectedIndex == -1 ) 
         {
         	Toolkit.getDefaultToolkit().beep();
         	return;
         }
         
         String selectedEndpoint = (String) listModel.getElementAt( selectedIndex );
         
         List<String> list = new ArrayList<String>( Arrays.asList( iface.getEndpoints()) );
         list.add( 0, ALL_REQUESTS );
         list.add( 1, ALL_TEST_REQUESTS );
         list.add( 2, ALL_REQUESTS_AND_TEST_REQUESTS );
         list.add( 3, ALL_REQUESTS_WITH_NO_ENDPOINT );
         
         Object endpoint = UISupport.prompt(	"Assign selected endpoint to..", "Assign Endpoint", 
         		 list.toArray(), ALL_REQUESTS_WITH_NO_ENDPOINT );
         
         if( endpoint == null )
         	return;
         
         int changeCount = 0;
         
         if( endpoint.equals( ALL_REQUESTS ) || endpoint.equals( ALL_REQUESTS_WITH_NO_ENDPOINT )
         			|| endpoint.equals( ALL_REQUESTS_AND_TEST_REQUESTS ))
         {
	         for( int c = 0; c < iface.getOperationCount(); c++ )
	         {
	         	Operation operation = iface.getOperationAt( c );
	         	for( int i = 0; i < operation.getRequestCount(); i++ )
	         	{
	         		Request request = operation.getRequestAt( i );
	         		String ep = request.getEndpoint();
	
	               if( endpoint.equals( ALL_REQUESTS ) || endpoint.equals( ALL_REQUESTS_AND_TEST_REQUESTS ) ||
	               	 ( endpoint.equals(  ALL_REQUESTS_WITH_NO_ENDPOINT ) && ep == null ) ||
	               	 ( ep.equals( endpoint )))
	               {
	               	request.setEndpoint( selectedEndpoint );
	               	changeCount++;
	               }
	         	}
	         }
         }
         
         if( endpoint.equals( ALL_REQUESTS_AND_TEST_REQUESTS ) || endpoint.equals( ALL_TEST_REQUESTS ))
         {
         	for( TestSuite testSuite : iface.getProject().getTestSuites() )
	         {
	         	for( TestCase testCase : testSuite.getTestCaseList())
	         	{
	         		for( TestStep testStep : testCase.getTestStepList())
	         		{
	         			if( testStep instanceof WsdlTestRequestStep )
	         			{
	         				WsdlTestRequest testRequest = ((WsdlTestRequestStep)testStep).getTestRequest();
	         				testRequest.setEndpoint( selectedEndpoint );
	         				changeCount++;
	         			}
	         		}
	         	}
	         }
         }
         
         log.info( "Assigned endpoint [" + selectedEndpoint + "] to " + changeCount + " Requests" );
      }
   }
   
   private class EditAction extends AbstractAction
   {
      public EditAction()
      {
         super( "Edit" );
         putValue( Action.SHORT_DESCRIPTION, "Edit the selected endpoint" );
      }
      
      public void actionPerformed(ActionEvent e)
      {
         int selectedIndex = list.getSelectedIndex();
         if( selectedIndex == -1 ) 
         {
         	Toolkit.getDefaultToolkit().beep();
         	return;
         }
         	
         String oldEndpoint = (String) listModel.getElementAt( selectedIndex );
         String newEndpoint = UISupport.prompt( "Edit endpoint address", "Edit Endpoint", oldEndpoint );
         if( newEndpoint == null ) return;
         
         listModel.setElementAt( newEndpoint, selectedIndex );
         iface.changeEndpoint( oldEndpoint, newEndpoint );
      }
   }
   
   private class DeleteAction extends AbstractAction
   {
      public DeleteAction()
      {
         super( "Delete" );
         putValue( Action.SHORT_DESCRIPTION, "Deletes the selected endpoint from the list" );
      }
      
      public void actionPerformed(ActionEvent e )
      {   
         int index = list.getSelectedIndex();
         if( index == -1)
         {
         	Toolkit.getDefaultToolkit().beep();
         	return;
         }
         
         if( UISupport.confirm( "Delete selected endpoint?", "Delete Endpoint" ))
         {
	         String oldEndpoint = (String) listModel.getElementAt( index );
	         listModel.removeElementAt( index );
	         iface.removeEndpoint( oldEndpoint );
         }
      }
   }

   private class OkAction extends AbstractAction
   {
      public OkAction()
      {
         super( "OK" );
         putValue( Action.SHORT_DESCRIPTION, "Closes this dialog" );
      }
      
      public void actionPerformed(ActionEvent e)
      {
         dialog.setVisible( false );
      }
   }
}
