/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.loadtest;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import com.eviware.soapui.model.testsuite.LoadTestRunContext;
import com.eviware.soapui.model.testsuite.LoadTestRunner;

/**
 * LoadTestRunContext implementation for WsdlLoadTests
 * 
 * @author Ole.Matzura
 */

public class WsdlLoadTestContext implements LoadTestRunContext, Map<String,Object>
{
	private Map<String,Object> properties = Collections.synchronizedMap( new HashMap<String,Object>() );
	private final WsdlLoadTestRunner runner;
	
	public WsdlLoadTestContext(WsdlLoadTestRunner runner)
	{
		this.runner = runner;
	}

	public Object getProperty(String name)
	{
		return properties.get( name );
	}

	public void setProperty(String name, Object value)
	{
		properties.put( name, value );
	}

	public Map getProperties()
	{
		return properties;
	}

	public boolean hasProperty(String name)
	{
		return properties.containsKey( name );
	}

	public LoadTestRunner getLoadTestRunner()
	{
		return runner;
	}

	public void clear()
	{
		properties.clear();
	}

	public boolean containsKey( Object arg0 )
	{
		return properties.containsKey( arg0 );
	}

	public boolean containsValue( Object arg0 )
	{
		return properties.containsValue( arg0 );
	}

	public Set<Entry<String, Object>> entrySet()
	{
		return properties.entrySet();
	}

	public boolean equals( Object arg0 )
	{
		return properties.equals( arg0 );
	}

	public Object get( Object arg0 )
	{
		return properties.get( arg0 );
	}

	public int hashCode()
	{
		return properties.hashCode();
	}

	public boolean isEmpty()
	{
		return properties.isEmpty();
	}

	public Set<String> keySet()
	{
		return properties.keySet();
	}

	public Object put( String arg0, Object arg1 )
	{
		return properties.put( arg0, arg1 );
	}

	public void putAll( Map<? extends String, ? extends Object> arg0 )
	{
		properties.putAll( arg0 );
	}

	public Object remove( Object arg0 )
	{
		return properties.remove( arg0 );
	}

	public int size()
	{
		return properties.size();
	}

	public Collection<Object> values()
	{
		return properties.values();
	}

	
}
