/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.attachments;

import java.awt.Component;
import java.awt.Toolkit;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import javax.swing.AbstractListModel;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JTable;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlAttachmentPart;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.model.iface.Attachment;
import com.eviware.soapui.support.Tools;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.components.JXToolBar;

/**
 * Utility Panel for displaying a table of attachments
 * 
 * @author emibre
 */

public class AttachmentPanel extends javax.swing.JPanel
{
	private DropTarget dropTarget;
	private FileTransferHandler fileTransferHandler;
	private RequestAttachmentTableModel tableModel;
	private JFileChooser fc;
	private final WsdlRequest request;
	boolean allowChange = false;
	boolean isRequest = false;
	private JButton exportBtn;

	/** Creates new form FileTableList */
	public AttachmentPanel(WsdlRequest request, boolean isRequest)
	{
		this.request = request;
		this.allowChange = isRequest;
		this.isRequest = isRequest;
		initComponents();
		initFileTransfer();
	}
	
	public void release()
	{
		tableModel.release();
		if( attachmentPartCellEditor != null )
			attachmentPartCellEditor.release();
	}

	private void initFileTransfer()
	{
		if (allowChange)
		{
			fileTransferHandler = new FileTransferHandler(tableModel);
			fileTable.setDragEnabled(true);
			fileTable.setTransferHandler(fileTransferHandler);

			dropTarget = new DropTarget();
			dropTarget.setActive(true);
			try
			{
				dropTarget.addDropTargetListener(new DropTargetListener()
				{
					public void dragEnter(DropTargetDragEvent dtde)
					{
					}

					public void dragExit(DropTargetEvent dte)
					{
					}

					public void dragOver(DropTargetDragEvent dtde)
					{
					}

					@SuppressWarnings("unchecked")
					public void drop(DropTargetDropEvent dtde)
					{
						try
						{
							dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
							Transferable trans = dtde.getTransferable();
							List<File> files = (List<File>) trans.getTransferData(DataFlavor.javaFileListFlavor);
							for (File f : files)
							{
								System.out.println("Dropping file: " + f.getName());

								Boolean retval = UISupport.confirmOrCancel("Cache attachment in request?", "Att Attachment");
								if (retval == null)
									return;

								tableModel.addFile(f, retval);
							}

						}
						catch (Exception e)
						{
							SoapUI.logError( e );
						}
					}

					public void dropActionChanged(DropTargetDragEvent dtde)
					{
					}
				});
			}
			catch (Exception e)
			{
				SoapUI.logError( e );
			}

			jScrollPane1.getViewport().setDropTarget(dropTarget);
		}
	}

	private void initComponents()
	{
		jScrollPane1 = new javax.swing.JScrollPane();
		tableModel = new RequestAttachmentTableModel(request, isRequest);
		fileTable = new JTable(tableModel);

		if (isRequest)
		{
			attachmentPartCellEditor = new AttachmentPartCellEditor();
			fileTable.getColumnModel().getColumn(3).setCellEditor(attachmentPartCellEditor);
		}

		setLayout(new java.awt.BorderLayout());
		jScrollPane1.setViewportView(fileTable);

		add(jScrollPane1, java.awt.BorderLayout.CENTER);

		jPanel1 = UISupport.createToolbar();
		
		if (allowChange)
		{
			addFileBtn = new javax.swing.JButton();
			removeBtn = new javax.swing.JButton();

			addFileBtn.setText("Add file");
			addFileBtn.addActionListener(new java.awt.event.ActionListener()
			{
				public void actionPerformed(java.awt.event.ActionEvent evt)
				{
					addFileBtnActionPerformed(evt);
				}
			});

			jPanel1.addFixed(addFileBtn);

			removeBtn.setText("Remove selected");
			removeBtn.setEnabled(false);
			removeBtn.addActionListener(new java.awt.event.ActionListener()
			{
				public void actionPerformed(java.awt.event.ActionEvent evt)
				{
					removeBtnActionPerformed(evt);
				}
			});

			jPanel1.addFixed(removeBtn);
		}

		exportBtn = new javax.swing.JButton();
		exportBtn.setText("Export selected");
		exportBtn.setEnabled(false);
		exportBtn.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				exportBtnActionPerformed(evt);
			}
		});

		jPanel1.addFixed(exportBtn);
		jPanel1.addGlue();
		jPanel1.addFixed( UISupport.createToolbarButton( new ShowOnlineHelpAction(HelpUrls.ATTACHMENTS_HELP_URL)));
		add(jPanel1, java.awt.BorderLayout.SOUTH);

		fileTable.getSelectionModel().addListSelectionListener(new ListSelectionListener()
		{
			public void valueChanged(ListSelectionEvent e)
			{
				if( removeBtn != null )
					removeBtn.setEnabled(fileTable.getSelectedRowCount() > 0);
				
				exportBtn.setEnabled(fileTable.getSelectedRowCount() > 0);
			}
		});
		
		fileTable.addMouseListener(new MouseAdapter()
		{
			public void mouseClicked(MouseEvent e)
			{
				if (e.getClickCount() < 2)
					return;

				int ix = fileTable.getSelectedRow();
				if (ix == -1)
					return;

				Attachment attachment = isRequest ? request.getAttachmentAt(ix)
						: request.getResponse().getAttachments()[ix];
				String url = attachment.getUrl();
				if (url != null)
				{
					Tools.openURL(url);
				}
				else
				{
					Toolkit.getDefaultToolkit().beep();
				}
			}
		});
	}

	protected void exportBtnActionPerformed( ActionEvent evt )
	{
		File file = UISupport.getFileDialogs().saveAs( this, "Export Attachment.." );
		while( file != null && file.exists() && 
				 !UISupport.confirm( "File " + file.getName() + " exists, overwrite?", "Export Attachment" ))
		{
			file = UISupport.getFileDialogs().saveAs( this, "Export Attachment.." );
		}
		
		if( file != null )
		{
			Attachment attachment = tableModel.getAttachmentAt(fileTable.getSelectedRow());
			try
			{
				FileOutputStream out = new FileOutputStream( file );
				
				long total = Tools.writeAll( out, attachment.getInputStream() );
				out.close();
				UISupport.showInfoMessage( "Written [" + total + "] bytes to " + file.getName() );
			}
			catch( Exception e )
			{
				UISupport.showErrorMessage( e );
			}
		}
	}

	private void addFileBtnActionPerformed(java.awt.event.ActionEvent evt)
	{// GEN-FIRST:event_addFileBtnActionPerformed
		if (fc == null)
			fc = new JFileChooser();

		int returnVal = fc.showOpenDialog(this);

		if (returnVal == JFileChooser.APPROVE_OPTION)
		{
			File file = fc.getSelectedFile();
			Boolean retval = UISupport.confirmOrCancel("Cache attachment in request?", "Att Attachment");
			if (retval == null)
				return;
			try
			{
				tableModel.addFile(file, retval);
			}
			catch (IOException e)
			{
				UISupport.showErrorMessage(e);
			}
		}
		else
		{
			System.out.println("Open command cancelled by user.");
		}
	}// GEN-LAST:event_addFileBtnActionPerformed

	private void removeBtnActionPerformed(java.awt.event.ActionEvent evt)
	{// GEN-FIRST:event_removeBtnActionPerformed
		if (UISupport.confirm("Remove selected attachments?", "Remove Attachments"))
			tableModel.removeAttachment(fileTable.getSelectedRows());
	}// GEN-LAST:event_removeBtnActionPerformed

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JButton addFileBtn;
	private JTable fileTable;
	private JXToolBar jPanel1;
	private javax.swing.JScrollPane jScrollPane1;
	private javax.swing.JButton removeBtn;
	private AttachmentPartCellEditor attachmentPartCellEditor;

	// End of variables declaration//GEN-END:variables

	private class AttachmentPartCellEditor extends DefaultCellEditor
	{
		public AttachmentPartCellEditor()
		{
			super(new JComboBox(new PartsComboBoxModel()));
		}

		public void release()
		{
			((PartsComboBoxModel) ((JComboBox) editorComponent).getModel()).release();
		}

		public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column)
		{
			((PartsComboBoxModel) ((JComboBox) editorComponent).getModel()).init(tableModel.getAttachmentAt(row));
			return super.getTableCellEditorComponent(table, value, isSelected, row, column);
		}
	}

	private final class PartsComboBoxModel extends AbstractListModel implements ComboBoxModel, PropertyChangeListener
	{
		private Attachment attachment;
		private WsdlAttachmentPart[] parts;
		
		public PartsComboBoxModel()
		{
			request.addPropertyChangeListener( this );
		}

		public void release()
		{
			request.removePropertyChangeListener( this );
		}

		public void init(Attachment attachment)
		{
			System.out.println( "Initializing parts..");
			this.attachment = attachment;
			parts = request.getDefinedAttachmentParts();
		}

		public Object getElementAt(int index)
		{
			return parts == null ? null : parts[index].getName();
		}

		public int getSize()
		{
			return parts == null ? 0 : parts.length;
		}

		public Object getSelectedItem()
		{
			return attachment == null ? null : attachment.getPart();
		}

		public void setSelectedItem(Object anItem)
		{
			if (attachment != null)
				attachment.setPart((String) anItem);
		}

		public void propertyChange( PropertyChangeEvent arg0 )
		{
			if( arg0.getPropertyName().equals( WsdlRequest.ATTACHMENTS_PROPERTY ))
			{
				// delete our current one?
				if( arg0.getOldValue() == attachment && arg0.getNewValue() == null )
				{
					attachment = null;
					parts = null;
				}
			}
		}
	}
}
