/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.attachments;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;

import javax.swing.table.AbstractTableModel;

import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.support.WsdlAttachment;
import com.eviware.soapui.model.iface.Attachment;

/**
 * TableModel for Request Attachments
 * 
 * @author emibre
 */

public class RequestAttachmentTableModel extends AbstractTableModel implements PropertyChangeListener,
			AttachmentTableModel
{

	WsdlRequest request;
	boolean isRequest = false;

	/** Creates a new instance of AttachmentTableModel */
	public RequestAttachmentTableModel( WsdlRequest request, boolean isRequest )
	{
		this.request = request;
		this.isRequest = isRequest;

		this.request.addPropertyChangeListener( this );
	}

	public void release()
	{
		request.removePropertyChangeListener( this );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.eviware.soapui.impl.wsdl.panels.attachments.AttachmentTableModel#addFile(java.io.File,
	 *      boolean)
	 */
	public void addFile( File file, boolean cacheInRequest ) throws IOException
	{
		if( isRequest )
		{
			Attachment attachment = request.attachFile( file, cacheInRequest );
			attachment.setContentType( ContentTypeHandler.getContentTypeFromFilename( file.getName() ) );
		}

		this.fireTableRowsInserted( request.getAttachmentCount(), request.getAttachmentCount() );
	}

	public void removeAttachment( int[] rowIndexes )
	{
		Arrays.sort( rowIndexes );
		for( int i = rowIndexes.length - 1; i >= 0; i-- )
			removeAttachment( rowIndexes[i] );
	}

	public void removeAttachment( int rowIndex )
	{
		if( isRequest )
		{
			request.removeAttachment( request.getAttachmentAt( rowIndex ) );
			this.fireTableRowsDeleted( rowIndex, rowIndex );
		}
	}

	public int getRowCount()
	{
		if( isRequest )
			return request.getAttachmentCount();
		else
		{
			try
			{
				// Response may not exist yet and attachments may be null
				return request.getResponse().getAttachments().length;
			}
			catch( Exception e )
			{
				return 0;
			}
		}
	}

	public int getColumnCount()
	{
		return isRequest ? 6 : 5;
	}

	public Attachment getAttachmentAt( int rowIndex )
	{
		if( isRequest )
			return request.getAttachmentAt( rowIndex );
		else
			return request.getResponse().getAttachments()[rowIndex];
	}

	public Object getValueAt( int rowIndex, int columnIndex )
	{
		if( rowIndex > getRowCount() )
			return null;

		Attachment att = null;
		if( isRequest )
			att = request.getAttachmentAt( rowIndex );
		else
			att = request.getResponse().getAttachments()[rowIndex];

		switch( columnIndex )
		{
		case 0:
			return att.getName();
		case 1:
			return att.getContentType();
		case 2:
			return att.getSize();
		case 3:
			return att.getPart();
		case 4:
			return att.getAttachmentType();
		case 5:
			return att.getContentID();
		default:
			return null;
		}
	}

	public int findColumn( String columnName )
	{
		if( columnName.equals( "Name" ) )
			return 0;
		else if( columnName.equals( "Content type" ) )
			return 1;
		else if( columnName.equals( "Size" ) )
			return 2;
		else if( columnName.equals( "Part" ) )
			return 3;
		else if( columnName.equals( "Type" ) )
			return 4;

		return -1;
	}

	public String getColumnName( int column )
	{
		if( column == 0 )
			return "Name";
		else if( column == 1 )
			return "Content type";
		else if( column == 2 )
			return "Size";
		else if( column == 3 )
			return "Part";
		else if( column == 4 )
			return "Type";
		else if( column == 5 )
			return "ContentID";
		else
			return null;
	}

	public boolean isCellEditable( int rowIndex, int columnIndex )
	{
		return isRequest && ( columnIndex == 1 || columnIndex == 3 || columnIndex == 5 );
	}

	public void setValueAt( Object aValue, int rowIndex, int columnIndex )
	{
		if( !isRequest )
			return;

		WsdlAttachment att = ( WsdlAttachment ) request.getAttachmentAt( rowIndex );
		if( columnIndex == 1 )
			att.setContentType( ( String ) aValue );
		else if( columnIndex == 3 )
			att.setPart( ( String ) aValue );
		else if( columnIndex == 5 )
			att.setContentID( ( String ) aValue );

		fireTableRowsUpdated( rowIndex, rowIndex );
	}

	/**
	 * Update table when attachments or response changes
	 */

	public void propertyChange( PropertyChangeEvent evt )
	{
		fireTableDataChanged();
	}
}
