/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.request.components.editor.inspectors.registry;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Registry of registered XmlInspectorFactories
 * 
 * @author ole.matzura
 */

public class XmlInspectorRegistry
{
	private static XmlInspectorRegistry instance;
	private List<XmlInspectorFactory> factories = new ArrayList<XmlInspectorFactory>();
	
	public XmlInspectorRegistry()
	{
	}
	
	public void addFactory( XmlInspectorFactory factory )
	{
		factories.add( factory );
	}

	public static final XmlInspectorRegistry getInstance()
	{
		if( instance == null )
			instance = new XmlInspectorRegistry();
		
		return instance;
	}
	
	public XmlInspectorFactory [] getFactories()
	{
		return factories.toArray( new XmlInspectorFactory[factories.size()] );
	}
	
	public XmlInspectorFactory [] getFactoriesOfType( Class type )
	{
		List<XmlInspectorFactory> result = new ArrayList<XmlInspectorFactory>();
		for( XmlInspectorFactory factory : factories )
		{
			if( Arrays.asList( factory.getClass().getInterfaces() ).contains( type )) 
				result.add( factory );
		}
		
		return result.toArray( new XmlInspectorFactory[result.size()] );
	}
}
