/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.testcase;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;

import com.eviware.soapui.impl.wsdl.testcase.TestCaseLogItem;
import com.eviware.soapui.impl.wsdl.testcase.TestCaseLogModel;
import com.eviware.soapui.model.testsuite.TestStepResult;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.swing.ActionList;
import com.eviware.soapui.support.action.swing.ActionSupport;

/**
 * Panel for displaying TestStepResults
 *  
 * @author Ole.Matzura
 */

public class TestCaseLog extends JPanel
{
	private TestCaseLogModel logListModel;
	private JList testLogList;

	public TestCaseLog()
	{
		super(new BorderLayout());

		buildUI();
	}

	private void buildUI()
	{
		logListModel = new TestCaseLogModel();
		testLogList = new JList(logListModel);
		testLogList.setCellRenderer(new TestLogCellRenderer());
		testLogList.setPrototypeCellValue( "Testing 123" );
		testLogList.setFixedCellWidth( -1 );
		testLogList.addMouseListener(new LogListMouseListener());
		testLogList.setBorder( BorderFactory.createLineBorder(  Color.GRAY ) );

		JScrollPane scrollPane = new JScrollPane(testLogList);
		scrollPane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(), "Test Log"));
		add(scrollPane, BorderLayout.CENTER);
	}

	private static final class TestLogCellRenderer extends JLabel implements ListCellRenderer
	{
		public TestLogCellRenderer()
		{
			setOpaque(true);
			setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3));
			setIcon(null);
		}
		
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected,
				boolean cellHasFocus)
		{
			if (value instanceof String)
			{
				setText(value.toString());
			}
			else if (value instanceof TestCaseLogItem)
			{
				TestCaseLogItem logItem = (TestCaseLogItem) value;
				String msg = logItem.getMsg();
				setText(msg == null ? "" : msg);
			}

			if (isSelected)
			{
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			}
			else
			{
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			}

			setEnabled(list.isEnabled());
			//setFont(list.getFont());

			return this;
		}
	}

	/**
	 * Mouse Listener for triggering default action and showing popup for log list items
	 * 
	 * @author Ole.Matzura
	 */

	private final class LogListMouseListener extends MouseAdapter
	{
		public void mouseClicked(MouseEvent e)
		{
			if (e.getClickCount() < 2)
				return;
			int selectedIndex = testLogList.getSelectedIndex();
			if (selectedIndex == -1)
				return;
			TestStepResult result = logListModel.getResultAt(selectedIndex);
			if (result != null && result.getActions() != null)
				result.getActions().performDefaultAction(new ActionEvent(this, 0, null));
		}

		public void mousePressed(MouseEvent e)
		{
			if (e.isPopupTrigger())
				showPopup(e);
		}

		public void mouseReleased(MouseEvent e)
		{
			if (e.isPopupTrigger())
				showPopup(e);
		}

		public void showPopup(MouseEvent e)
		{
			int row = testLogList.locationToIndex(e.getPoint());
			if (row == -1)
				return;

			if (testLogList.getSelectedIndex() != row)
			{
				testLogList.setSelectedIndex(row);
			}

			TestStepResult result = logListModel.getResultAt(row);
			if (result == null)
				return;

			ActionList actions = result.getActions();

			if (actions == null || actions.getActionCount() == 0)
				return;

			JPopupMenu popup = ActionSupport.buildPopup(actions);
			UISupport.showPopup(popup, testLogList, e.getPoint());
		}
	}

	public void clear()
	{
		logListModel.clear();
	}

	public void addText(String string)
	{
		logListModel.addText( string );
	}

	public void addTestStepResult(TestStepResult stepResult)
	{
		logListModel.addTestStepResult( stepResult );
	}

	public TestCaseLogModel getLogListModel()
	{
		return logListModel;
	}

	public void setLogListModel(TestCaseLogModel logListModel)
	{
		this.logListModel = logListModel;
		testLogList.setModel( logListModel );
	}
}
