/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.teststeps;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Box;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.text.Document;

import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.panels.support.TestRunComponentEnabler;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlPropertiesTestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestStepListenerAdapter;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlPropertiesTestStep.StepProperty;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.testsuite.TestStepProperty;
import com.eviware.soapui.support.DocumentListenerAdapter;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.components.JXToolBar;
import com.eviware.soapui.ui.desktop.DesktopPanel;

/**
 * DesktopPanel for WsdlPropertiesTestSteps
 * 
 * @author Ole.Matzura
 */

public class PropertiesStepDesktopPanel extends JPanel implements DesktopPanel
{
	private final WsdlPropertiesTestStep testStep;
	private JTextField sourceField;
	private JTextField targetField;
	private PropertiesModel propertiesModel;
	private JTable propertiesTable;
	private RemovePropertyAction removePropertyAction;
	private TestRunComponentEnabler componentEnabler;
	private InternalWsdlTestStepListener wsdlTestStepListener;

	public PropertiesStepDesktopPanel(WsdlPropertiesTestStep testStep)
	{
		super(new BorderLayout());
		this.testStep = testStep;
		componentEnabler = new TestRunComponentEnabler(testStep.getTestCase());
		buildUI();

		wsdlTestStepListener = new InternalWsdlTestStepListener();
		testStep.addTestStepListener(wsdlTestStepListener);
	}

	private void buildUI()
	{
		add(buildToolbar(), BorderLayout.NORTH);
		add(buildPropertiesTable(), BorderLayout.CENTER);

		setPreferredSize(new Dimension(600, 400));
	}

	private Component buildPropertiesTable()
	{
		propertiesModel = new PropertiesModel();
		propertiesTable = new JTable(propertiesModel);

		propertiesTable.setDragEnabled(true);
		
//		propertiesTable.setTransferHandler(new PropertiesTransferHandler( propertiesTable ));

		propertiesTable.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
		propertiesTable.getSelectionModel().addListSelectionListener(new ListSelectionListener()
		{

			public void valueChanged(ListSelectionEvent e)
			{
				removePropertyAction.setEnabled(propertiesTable.getSelectedRow() != -1);
			}
		});

		componentEnabler.add(propertiesTable);

		return new JScrollPane(propertiesTable);
	}

	private JComponent buildToolbar()
	{
		JXToolBar toolbar = UISupport.createToolbar();

		JButton addPropertyButton = UISupport.createToolbarButton(new AddPropertyAction());
		toolbar.add(addPropertyButton);
		removePropertyAction = new RemovePropertyAction();
		JButton removePropertyButton = UISupport.createToolbarButton(removePropertyAction);
		toolbar.add(removePropertyButton);

		toolbar.addSeparator();
		toolbar.add(new JLabel("Load from:"));
		sourceField = new JTextField(testStep.getSource(), 15);
		sourceField.setToolTipText("The filename/url or referring system-property to load properties from");
		sourceField.getDocument().addDocumentListener(new DocumentListenerAdapter()
		{
			public void update(Document document)
			{
				testStep.setSource(sourceField.getText());

			}
		});

		toolbar.addFixed(sourceField);
		JButton setSourceButton = UISupport.createToolbarButton(new SetPropertiesSourceAction());
		toolbar.add(setSourceButton);

		toolbar.addSeparator();
		toolbar.add(new JLabel("Save to:"));
		targetField = new JTextField(testStep.getTarget(), 15);
		targetField.setToolTipText("The filename/url or referring system-property to save properties to");
		targetField.getDocument().addDocumentListener(new DocumentListenerAdapter()
		{
			public void update(Document document)
			{
				testStep.setTarget(targetField.getText());

			}
		});

		toolbar.addFixed(targetField);
		JButton setTargetButton = UISupport.createToolbarButton(new SetPropertiesTargetAction());
		toolbar.add(setTargetButton);

		toolbar.add(Box.createHorizontalGlue());
		toolbar.addSeparator();
		toolbar.add(UISupport.createToolbarButton(new ShowOnlineHelpAction(HelpUrls.PROPERTIESSTEPEDITOR_HELP_URL)));

		componentEnabler.add(sourceField);
		componentEnabler.add(targetField);
		componentEnabler.add(setTargetButton);
		componentEnabler.add(setSourceButton);
		componentEnabler.add(addPropertyButton);
		componentEnabler.add(removePropertyButton);

		return toolbar;
	}

	public Icon getIcon()
	{
		return getModelItem().getIcon();
	}

	public ModelItem getModelItem()
	{
		return testStep;
	}

	public boolean onClose(boolean canCancel)
	{
		if( propertiesTable.isEditing() )
			propertiesTable.getCellEditor().stopCellEditing();
		
		componentEnabler.release();
		testStep.removeTestStepListener(wsdlTestStepListener);
		return true;
	}

	public JComponent getComponent()
	{
		return this;
	}

	public boolean dependsOn(ModelItem modelItem)
	{
		return modelItem == testStep || modelItem == testStep.getTestCase()
				|| modelItem == testStep.getTestCase().getTestSuite()
				|| modelItem == testStep.getTestCase().getTestSuite().getProject();
	}

	public String getTitle()
	{
		return testStep.getTestCase().getName() + " - " + testStep.getName();
	}
	
	public String getDescription()
	{
		return "Properties: [" + testStep.getName() + "] - " + testStep.getTestStepTitle();
	}

	private final class InternalWsdlTestStepListener extends WsdlTestStepListenerAdapter
	{
		private boolean enabled = true;

		public boolean isEnabled()
		{
			return enabled;
		}

		public void setEnabled(boolean enabled)
		{
			this.enabled = enabled;
		}

		public void propertyAdded(String name)
		{
			if (enabled)
				propertiesModel.fireTableDataChanged();
		}

		public void propertyRemoved(String name)
		{
			if (enabled)
				propertiesModel.fireTableDataChanged();
		}

		public void propertyRenamed(String oldName, String newName)
		{
			if (enabled)
				propertiesModel.fireTableDataChanged();
		}

		public void propertyValueChanged(String name, String oldValue, String newValue)
		{
			if (enabled)
				propertiesModel.fireTableDataChanged();
		}
	}

	private class PropertiesModel extends AbstractTableModel
	{
		public int getRowCount()
		{
			return testStep.getStepPropertyCount();
		}

		public int getColumnCount()
		{
			return 2;
		}

		public String getColumnName(int columnIndex)
		{
			switch (columnIndex)
			{
			case 0:
				return "Name";
			case 1:
				return "Value";
			}

			return null;
		}

		public boolean isCellEditable(int rowIndex, int columnIndex)
		{
			return true;
		}

		public void setValueAt(Object aValue, int rowIndex, int columnIndex)
		{
			StepProperty property = testStep.getTestStepPropertyAt(rowIndex);
			switch (columnIndex)
			{
			case 0:
			{
				TestStepProperty prop = testStep.getProperty((String) aValue);
				if (prop != null && prop != property)
				{
					UISupport.showErrorMessage("Property name exists!");
					return;
				}
				wsdlTestStepListener.setEnabled(false);
				property.setName(aValue.toString());
				wsdlTestStepListener.setEnabled(true);
				break;
			}
			case 1:
			{
				wsdlTestStepListener.setEnabled(false);
				property.setValue(aValue.toString());
				wsdlTestStepListener.setEnabled(true);
				break;
			}
			}
		}

		public Object getValueAt(int rowIndex, int columnIndex)
		{
			TestStepProperty property = testStep.getTestStepPropertyAt(rowIndex);

			switch (columnIndex)
			{
			case 0:
				return property.getName();
			case 1:
				return property.getValue();
			}

			return null;
		}
	}

	private class AddPropertyAction extends AbstractAction
	{
		public AddPropertyAction()
		{
			putValue(Action.SMALL_ICON, UISupport.createImageIcon("/add_property.gif"));
			putValue(Action.SHORT_DESCRIPTION, "Adds a property to the property list");
		}

		public void actionPerformed(ActionEvent e)
		{
			String name = UISupport.prompt("Specify property name", "Add Property", "");
			if (name != null)
			{
				wsdlTestStepListener.setEnabled(false);
				testStep.addProperty(name);
				wsdlTestStepListener.setEnabled(true);
				int row = testStep.getStepPropertyCount() - 1;
				propertiesModel.fireTableRowsInserted(row, row);
				propertiesTable.editCellAt(row, 1);
			}
		}
	}

	private class RemovePropertyAction extends AbstractAction
	{
		public RemovePropertyAction()
		{
			putValue(Action.SMALL_ICON, UISupport.createImageIcon("/remove_property.gif"));
			putValue(Action.SHORT_DESCRIPTION, "Removes the selected property from the property list");
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e)
		{
			int row = propertiesTable.getSelectedRow();
			if (row == -1)
				return;

			UISupport.stopCellEditing(propertiesTable);

			if (UISupport.confirm("Remove property [" + propertiesModel.getValueAt(row, 0) + "]?", "Remove Property"))
			{
				wsdlTestStepListener.setEnabled(false);
				testStep.removePropertyAt(row);
				wsdlTestStepListener.setEnabled(true);
				propertiesModel.fireTableRowsDeleted(row, row);
			}
		}
	}

	private class SetPropertiesSourceAction extends AbstractAction
	{
		public SetPropertiesSourceAction()
		{
			putValue(Action.SMALL_ICON, UISupport.createImageIcon("/set_properties_source.gif"));
			putValue(Action.SHORT_DESCRIPTION, "Selects the properties source file");
		}

		public void actionPerformed(ActionEvent e)
		{
			File file = UISupport.getFileDialogs().open(this, "Set properties source", null, null, null);
			if (file != null)
			{
				testStep.setSource(file.getAbsolutePath());
				sourceField.setText(testStep.getSource());
				try
				{
					boolean createMissing = UISupport.confirm("Create missing properties?", "Set Properties Source");
					int cnt = testStep.loadProperties(createMissing);
					UISupport.showInfoMessage("Loaded " + cnt + " properties from [" + testStep.getSource() + "]");
				}
				catch (IOException e1)
				{
					UISupport.showErrorMessage("Failed to load properties from [" + testStep.getSource() + "]; " + e1);
				}
			}
		}
	}

	private class SetPropertiesTargetAction extends AbstractAction
	{
		public SetPropertiesTargetAction()
		{
			putValue(Action.SMALL_ICON, UISupport.createImageIcon("/set_properties_target.gif"));
			putValue(Action.SHORT_DESCRIPTION, "Selects the properties target file");
		}

		public void actionPerformed(ActionEvent e)
		{
			File file = UISupport.getFileDialogs().saveAs(this, "Set properties target");
			if (file != null)
			{
				testStep.setTarget(file.getAbsolutePath());
				targetField.setText(testStep.getTarget());

				try
				{
					int cnt = testStep.saveProperties();
					UISupport.showInfoMessage("Saved " + cnt + " properties to [" + testStep.getTarget() + "]");
				}
				catch (IOException e1)
				{
					UISupport.showErrorMessage("Failed to save properties to [" + testStep.getTarget() + "]; " + e1);
				}
			}
		}
	}

	/*
	public class PropertiesTransferHandler extends AbstractPropertiesTransferHandler
	{
		public PropertiesTransferHandler(JComponent component)
		{
			super(component);
		}

		protected StepProperty getSelectedProperty(JComponent c)
		{
			int rowIndex = propertiesTable.getSelectedRow();
			if (rowIndex == -1)
				return null;

			return testStep.getTestStepPropertyAt(rowIndex);
		}
	}*/
}
