/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.testsuite;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JToggleButton;
import javax.swing.text.Document;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlTestSuite;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.support.PropertiesMap;
import com.eviware.soapui.model.support.TestRunListenerAdapter;
import com.eviware.soapui.model.support.TestSuiteListenerAdapter;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestRunContext;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestSuite.TestSuiteRunType;
import com.eviware.soapui.support.DocumentListenerAdapter;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.components.JUndoableTextArea;
import com.eviware.soapui.support.components.JXToolBar;
import com.eviware.soapui.ui.desktop.DesktopPanel;

/**
 * DesktopPanel for WsdlTestSuite
 * 
 * @author Ole.Matzura
 */

@SuppressWarnings("serial")
public class WsdlTestSuiteDesktopPanel extends JPanel implements DesktopPanel
{
	private final WsdlTestSuite testSuite;
	private JProgressBar progressBar;
	private JTestCaseList testCaseList;
	private RunAction runAction = new RunAction();
	private CancelAction cancelAction = new CancelAction();
	private TestSuiteRunner testSuiteRunner = new TestSuiteRunner();
	private JToggleButton sequentialButton;
	private JToggleButton parallellButton;
	private final InternalTestSuiteListener testSuiteListener = new InternalTestSuiteListener();
	private JTextArea descriptionArea;

	public WsdlTestSuiteDesktopPanel(WsdlTestSuite testSuite)
	{
		super( new BorderLayout() );
		this.testSuite = testSuite;
		
		buildUI();
		testSuite.addTestSuiteListener( testSuiteListener );
	}

	private void buildUI()
	{
		add( buildToolbar(), BorderLayout.NORTH );
		add( buildContent(), BorderLayout.CENTER );
		
		setPreferredSize( new Dimension( 300, 400 ));
	}
	
	protected JTestCaseList getTestCaseList()
	{
		return testCaseList;
	}

	@Override
	public void addNotify()
	{
		super.addNotify();
		testSuite.addTestSuiteListener( testSuiteListener );
	}

	@Override
	public void removeNotify()
	{
		super.removeNotify();
		testSuite.removeTestSuiteListener( testSuiteListener );
	}

	private JComponent buildToolbar()
	{
		cancelAction.setEnabled( false );
		runAction.setEnabled( testSuite.getTestCaseCount() > 0 );
		
		JXToolBar toolbar = UISupport.createToolbar();
		
		addToolbarActions( toolbar );
		toolbar.add( Box.createHorizontalGlue() );
		toolbar.add(  UISupport.createToolbarButton( new ShowOnlineHelpAction( HelpUrls.TESTSUITEEDITOR_HELP_URL )));
		
		progressBar = new JProgressBar( 0, testSuite.getTestCaseCount() );
		JPanel progressPanel = UISupport.createProgressBarPanel(progressBar, 10, false );
		
		JPanel panel = new JPanel( new BorderLayout() );
	   
      panel.add( toolbar, BorderLayout.PAGE_START );
      panel.add( progressPanel, BorderLayout.CENTER );
      
		return panel;
	}

	protected void addToolbarActions( JXToolBar toolbar )
	{
		toolbar.add( UISupport.createToolbarButton( runAction ));
		toolbar.addRelatedGap();
		toolbar.add( UISupport.createToolbarButton( cancelAction ));
		
		ButtonGroup buttonGroup = new ButtonGroup();
		
		sequentialButton = new JToggleButton( UISupport.createImageIcon( "/sequential.gif" ), true );
		sequentialButton.setToolTipText( "The selected TestCases are run in sequence" );
		sequentialButton.setPreferredSize( UISupport.getPreferredButtonSize());
		sequentialButton.setSelected( testSuite.getRunType() == TestSuiteRunType.SEQUENTIAL );
		sequentialButton.addActionListener( new ActionListener() {

			public void actionPerformed(ActionEvent e)
			{
				testSuite.setRunType( TestSuiteRunType.SEQUENTIAL );
			}} );
		
		buttonGroup.add( sequentialButton );
		
		parallellButton = new JToggleButton( UISupport.createImageIcon( "/parallell.gif" ));
		parallellButton.setToolTipText( "The selected TestCases are run in parallel" );
		parallellButton.setPreferredSize( UISupport.getPreferredButtonSize());
		parallellButton.setSelected( testSuite.getRunType() == TestSuiteRunType.PARALLEL );
		parallellButton.addActionListener( new ActionListener() {

			public void actionPerformed(ActionEvent e)
			{
				testSuite.setRunType( TestSuiteRunType.PARALLEL );
			}} );
		
		buttonGroup.add( parallellButton );
		
		toolbar.addUnrelatedGap();
		toolbar.add( sequentialButton );
		toolbar.addRelatedGap();
		toolbar.add( parallellButton );
		
	}
	
	private JComponent buildContent()
	{
		JTabbedPane tabs = new JTabbedPane( JTabbedPane.TOP );
		
		testCaseList = buildTestCaseList( testSuite );
		
      tabs.addTab( "TestCases",  new JScrollPane( testCaseList ));
      tabs.addTab( "Description", buildDescriptionPanel() );
      
      return UISupport.createTabPanel( tabs, true );
	}

	private Component buildDescriptionPanel()
	{
   	JPanel panel = new JPanel( new BorderLayout() );
   	descriptionArea = new JUndoableTextArea( testSuite.getDescription() );
   	descriptionArea.getDocument().addDocumentListener( new DocumentListenerAdapter() 
   	{
			public void update(Document document)
			{
				testSuite.setDescription( descriptionArea.getText() );
			}} );
   	
   	panel.setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2));
   	panel.add( new JScrollPane( descriptionArea ), BorderLayout.CENTER );
   	
		return panel;
	}
	
	protected JTestCaseList buildTestCaseList(WsdlTestSuite testSuite)
	{
		return new JTestCaseList( testSuite );
	}

	public Icon getIcon()
	{
		return getModelItem().getIcon();
	}

	public WsdlTestSuite getModelItem()
	{
		return testSuite;
	}

	public boolean onClose( boolean canCancel )
	{
		return true;
	}

	public JComponent getComponent()
	{
		return this;
	}

	public boolean dependsOn(ModelItem modelItem)
	{
		return modelItem == testSuite || modelItem == testSuite.getProject();
	}

	public String getTitle()
	{
		return getModelItem().getName();
	}
	
	public String getDescription()
	{
		return "TestSuite: [" + getModelItem().getName() + "]";
	}
	
	protected void runTestSuite()
	{
		new Thread( testSuiteRunner, testSuite.getName() + " TestSuiteRunner" ).start();
	}

	protected void beforeRun()
	{
		runAction.setEnabled( false );
		cancelAction.setEnabled( true );
		testCaseList.setEnabled( false );
	}

	protected void afterRun()
	{
		runAction.setEnabled( true );
		cancelAction.setEnabled( false );
		testCaseList.setEnabled( true );
		
		progressBar.setString( "Finished" );
	}
	
	private final class InternalTestSuiteListener extends TestSuiteListenerAdapter
	{
		public void testCaseAdded(TestCase testCase)
		{
			runAction.setEnabled( testSuite.getTestCaseCount() > 0 );
		}

		public void testCaseRemoved(TestCase testCase)
		{
			runAction.setEnabled( testSuite.getTestCaseCount() > 0 );
		}
	}

	private class RunAction extends AbstractAction
	{
		public RunAction()
		{
			putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/run_testcase.gif" ));
			putValue( Action.SHORT_DESCRIPTION, "Runs the selected TestCases" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			runTestSuite();
		}
	}
	
	private class CancelAction extends AbstractAction
	{
		public CancelAction()
		{
			putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/stop_testcase.gif" ));
			putValue( Action.SHORT_DESCRIPTION, "Cancels ongoing TestCase runs" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
		   testSuiteRunner.cancel();
		}
	}
	
	/**
	 * Runs the selected testsuites..
	 * 
	 * @author Ole.Matzura
	 */
	
	public class TestSuiteRunner implements Runnable
	{
		private boolean canceled;
		private List<TestRunner> runners = new ArrayList<TestRunner>();
		private InternalTestRunListener internalTestRunListener = new InternalTestRunListener();
		
		public void cancel()
		{
			canceled = true;
			
			for( TestRunner runner : runners )
			{
				runner.cancel( "Canceled from TestSuite" );
			}
		}
		
		public void run()
		{
			canceled = false;
			beforeRun();
			
			int[] indices = testCaseList.getSelectedIndices();
			if( indices.length == 0 )
			{
				indices = new int[testSuite.getTestCaseCount()];
				for( int c = 0; c < indices.length; c++ )
					indices[c] = c;
			}
			
			progressBar.setValue( 0 );
			progressBar.setString( "" ); 
			progressBar.setMaximum( indices.length );
			
			TestSuiteRunType runType = testSuite.getRunType();
			
			for( int c = 0; c < indices.length; c++ )
			{
				TestCase testCase = (TestCase) testSuite.getTestCaseAt( indices[c] );
				if( SoapUI.getTestMonitor().hasRunningLoadTest( testCase ))
				{
					progressBar.setString( "Skipping " + testCase.getName() );
					progressBar.setValue( c+1 );
					continue;
				}
				
				if( runType == TestSuiteRunType.PARALLEL )
				{
					testCase.addTestRunListener( internalTestRunListener );
					progressBar.setString( "Starting " + testCase.getName() );
				}
				else
				{
					progressBar.setString( "Running " + testCase.getName() );
				}
				
				TestRunner runner = testCase.run( PropertiesMap.EMPTY_MAP, true );
				runners.add( runner );
				
				if( runType == TestSuiteRunType.SEQUENTIAL )
				{
					runner.waitUntilFinished();
					progressBar.setValue( c+1 );
					runners.remove( runner );
				}
				
				if( canceled )
					break;
			}

			if( runners.isEmpty() )
				afterRun();
		}
		
		/**
		 * Waits for running tests to finish when running in parallel
		 */
		
		private class InternalTestRunListener extends TestRunListenerAdapter
		{
			public void afterRun(TestRunner testRunner, TestRunContext runContext)
			{
				runners.remove( testRunner );
				testRunner.getTestCase().removeTestRunListener( this );
				
				progressBar.setValue( progressBar.getValue()+1 );
				
				if( runners.isEmpty() )
					WsdlTestSuiteDesktopPanel.this.afterRun();
			}
		}
	}
}
