/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.support;

import java.io.File;

import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.model.testsuite.TestRunContext;
import com.eviware.soapui.model.testsuite.TestStep;

public final class GroovyUtils
{
	private final TestRunContext context;

	public GroovyUtils( TestRunContext context )
	{
		this.context = context;
	}
	
	public String getProjectPath()
	{
		WsdlProject project = ( WsdlProject ) context.getTestCase().getTestSuite().getProject();
		String path = project.getPath();
		int ix = path.lastIndexOf( File.separatorChar );
		return ix == -1 ? path : path.substring( 0, ix );
	}
	
	public XmlHolder getXmlHolder( String xmlPropertyOrString ) throws Exception
	{
		Object property = context.getProperty( xmlPropertyOrString );
		if( property != null )
			return new XmlHolder( context, xmlPropertyOrString );
		else
			return new XmlHolder( xmlPropertyOrString );
	}
	
	public void setPropertyValue( String testStep, String property, String value ) throws Exception
	{
		TestStep step = context.getTestCase().getTestStepByName( testStep );
		if( step != null )
		{
			step.setPropertyValue( property, value );
		}
		else
		{
			throw new Exception( "Missing TestStep [" + testStep + "] in TestCase" );
		}
	}
}
