/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.x.form.support;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JCheckBox;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;

import com.eviware.x.form.XFormOptionsField;
import com.eviware.x.impl.swing.AbstractSwingXFormField;

/**
 * Swing-Specific multi-select list
 * 
 * @author ole.matzura
 */

public class XFormMultiSelectList extends AbstractSwingXFormField<JPanel> implements XFormOptionsField
{
	private JList list;
	private DefaultListModel listModel;
	private List<Boolean> selected = new ArrayList<Boolean>();

	public XFormMultiSelectList( String [] values )
	{
		super( new JPanel( new BorderLayout() ) );
		
		listModel = new DefaultListModel();
		for( String value : values )
		{
			selected.add( false );
			listModel.addElement( value );
		}
		
		list = new JList( listModel );
		list.setCellRenderer( new CheckListCellRenderer() );
		list.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		list.addMouseListener(new MouseAdapter()
      {
         public void mousePressed(MouseEvent e)
         {
            int index = list.locationToIndex(e.getPoint());

            if (index != -1) 
            {
            	selected.set( index, !selected.get(  index ));
               list.repaint();
            }
         }
      });
		
		getComponent().add( new JScrollPane( list ), BorderLayout.CENTER );
		getComponent().setSize( new Dimension( 400, 120 ) );
		getComponent().setMaximumSize( new Dimension( 400, 120 ) );
		getComponent().setPreferredSize( new Dimension( 400, 120 ) );
		getComponent().setMinimumSize( new Dimension( 400, 120 ) );
	}

	public String getValue()
	{
		return ( String ) list.getSelectedValue();
	}

	public void setValue( String value )
	{
		int index = listModel.indexOf( value );
		selected.set( index, true );
		list.setSelectedIndex( index );
	}

	public void addItem( String value )
	{
		listModel.addElement( value );
		selected.add(  false );
	}

	public String[] getOptions()
	{
		String [] options = new String[listModel.size()];
		for( int c = 0; c < options.length; c++ )
			options[c] = ( String ) listModel.get( c );
		return options;
	}

	public String[] getSelectedOptions()
	{
		List<String> result = new ArrayList<String>();
		
		for( int c = 0; c < selected.size(); c++ )
		{
			if( selected.get( c ))
				result.add( ( String ) listModel.get( c ));
		}
		
		return result.toArray( new String[result.size()] );
	}

	public void setOptions( Object[] values )
	{
		listModel.clear();
		selected.clear();
		for( Object value : values )
		{
			selected.add( false );
			listModel.addElement( value );
		}
	}
	
	public class CheckListCellRenderer extends JCheckBox implements ListCellRenderer
	{
		public CheckListCellRenderer()
		{
			setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3  ) );
		}
		
		public Component getListCellRendererComponent( JList list, Object value, int index, boolean isSelected,
					boolean cellHasFocus )
		{
			setText( value.toString() );
			setSelected( selected.get( index ) );
			
			if (isSelected)
			{
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			}
			else
			{
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			}
			
			return this;
		}
	}

	public void setSelectedOptions( String[] options )
	{
		List<String> asList = Arrays.asList( options );
		
		for( int c = 0; c < selected.size(); c++ )
		{
			selected.set( c, asList.contains( listModel.get( c )));
		}
		
		list.repaint();
	}
}
